#include "Cube.hpp"

#include <glm-0.9.5.2\glm\glm\gtc\matrix_transform.hpp>

using namespace cgue;
using namespace cgue::scene;

// Konstruktor
Cube::Cube(glm::mat4& matrix, Shader* _shader) : SceneObject(matrix), shader(_shader)
{
	// Load Data to buffer
	// pro attribut einen buffer


	// POSITION BUFFER
	glGenBuffers(1,&positionBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, positionBuffer);
	glBufferData(GL_ARRAY_BUFFER, CUBE_VERTEX_COUNT * 3 * sizeof(float), positions, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	// NORMAL BUFFER
	glGenBuffers(1,&normalBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, normalBuffer);
	glBufferData(GL_ARRAY_BUFFER, CUBE_VERTEX_COUNT * 3 * sizeof(float), normals, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	// UV BUFFER
	glGenBuffers(1,&uvBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, uvBuffer);
	glBufferData(GL_ARRAY_BUFFER, CUBE_VERTEX_COUNT * 2 * sizeof(float), uvs, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	// INDEX BUFFER
	glGenBuffers(1,&indexBuffer);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexBuffer);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, CUBE_INDEX_COUNT * sizeof(unsigned int), indices, GL_STATIC_DRAW);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

	glGenVertexArrays(1, &vao);
	glBindVertexArray(vao);



	// Bind Buffers - Buffers auf eine Variable schreiben

	// POSITION BUFFER
	glBindBuffer(GL_ARRAY_BUFFER, positionBuffer);
	GLint positionIndex = glGetAttribLocation(shader->programHandle, "position");
	glEnableVertexAttribArray(positionIndex);
	glVertexAttribPointer(positionIndex, 3, GL_FLOAT, GL_FALSE, 0, 0);

	// NORMAL BUFFER
	glBindBuffer(GL_ARRAY_BUFFER, normalBuffer);
	GLint normalIndex = glGetAttribLocation(shader->programHandle, "normal");
	glEnableVertexAttribArray(normalIndex);
	glVertexAttribPointer(normalIndex, 3, GL_FLOAT, GL_FALSE, 0, 0);

	// UV BUFFER
	glBindBuffer(GL_ARRAY_BUFFER, uvBuffer);
	GLint uvIndex = glGetAttribLocation(shader->programHandle, "uv");
	glEnableVertexAttribArray(uvIndex);
	glVertexAttribPointer(uvIndex, 2, GL_FLOAT, GL_FALSE, 0, 0);

	// INDEXBUFFER
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexBuffer);

	glBindVertexArray(0);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	
    modelMatrix = glm::translate(modelMatrix, glm::vec3( 10.0f , 3.0f, -190.0));

}

Cube::~Cube() {
	
	glDeleteBuffers(1, &positionBuffer);
	glDeleteBuffers(1, &indexBuffer);
	glDeleteBuffers(1, &normalBuffer);
	glDeleteBuffers(1, &uvBuffer);
	glDeleteVertexArrays(1, &vao);
}

void Cube::update(double deltaT, std::string tastatur) {
	//modelMatrix = glm::rotate(modelMatrix, 90.0f * (float)deltaT, glm::vec3(0.0f, 1.0f, 0.0f));
}

void Cube::draw() {

	glBindVertexArray(vao);

	glDrawElements(GL_TRIANGLES, CUBE_INDEX_COUNT, GL_UNSIGNED_INT, 0);

	glBindVertexArray(0);
}

const float Cube::positions[CUBE_VERTEX_COUNT * 3] = {


	// Back
	-50.0f, -450.0f, -150.0f,
	-50.0f, -350.0f, -150.0f,
	50.0f, -350.0f, -150.0f, 
	50.0f, -450.0f, -150.0f,

	// Front
	-50.0f, -450.0f, 50.0f,
	50.0f, -450.0f, 50.0f,
	50.0f, -350.0f, 50.0f,
	-50.0f, -350.0f, 50.0f,

	//Top
	50.0f, -350.0f, 50.0f,
	50.0f, -350.0f, -150.0f,
	-50.0f, -350.0f, -150.0f,
	-50.0f, -350.0f, 50.0f,

	//Bottom
	50.0f, -450.0f, 50.0f,
	-50.0f, -450.0f, 50.0f,
	-50.0f, -450.0f, -50.0f,
	50.0f, -450.0f, -50.0f,

	//Right
	50.0f, -350.0f, 50.0f,
	50.0f, -450.0f, 50.0f,
	50.0f, -450.0f, -150.0f,
	50.0f, -350.0f, -150.0f, 

	//Left
	-50.0f, -350.0f, 50.0f,
	-50.0f, -350.0f, -150.0f,
	-50.0f, -450.0f, -150.0f,
	-50.0f, -450.0f, 50.0f,
	
};


const float Cube::normals[CUBE_VERTEX_COUNT * 3] = {
	
	0.0f, 1.0f, 0.0f,
	0.0f, 1.0f, 0.0f,
	0.0f, 1.0f, 0.0f,
	0.0f, 1.0f, 0.0f,

	0.0f, -1.0f, 0.0f,
	0.0f, -1.0f, 0.0f,
	0.0f, -1.0f, 0.0f,
	0.0f, -1.0f, 0.0f,

	1.0f, 0.0f, 0.0f,
	1.0f, 0.0f, 0.0f,
	1.0f, 0.0f, 0.0f,
	1.0f, 0.0f, 0.0f,

	-1.0f, 0.0f, 0.0f,
	-1.0f, 0.0f, 0.0f,
	-1.0f, 0.0f, 0.0f,
	-1.0f, 0.0f, 0.0f,

	0.0f, 0.0f, -1.0f,
	0.0f, 0.0f, -1.0f,
	0.0f, 0.0f, -1.0f,
	0.0f, 0.0f, -1.0f,

	0.0f, 0.0f, 1.0f,
	0.0f, 0.0f, 1.0f,
	0.0f, 0.0f, 1.0f,
	0.0f, 0.0f, 1.0f,
};

const float Cube::uvs[CUBE_VERTEX_COUNT * 2] = {

	1.0f, 0.0f,
	1.0f, 1.0f,
	0.0f, 1.0f,
	0.0f, 0.0f,

	0.0f, 0.0f,
	1.0f, 0.0f,
	1.0f, 1.0f,
	0.0f, 1.0f,

	0.0f, 0.0f,
	1.0f, 0.0f,
	1.0f, 1.0f,
	0.0f, 1.0f,

	0.0f, 0.0f,
	1.0f, 0.0f,
	1.0f, 1.0f,
	0.0f, 1.0f,

	1.0f, 1.0f,
	1.0f, 0.0f,
	0.0f, 0.0f,
	0.0f, 1.0f,

	0.0f, 1.0f,
	1.0f, 1.0f,
	1.0f, 0.0f,
	0.0f, 0.0f

};

const unsigned int Cube::indices[CUBE_INDEX_COUNT] = {

	0,1,2,
	0,2,3,

	4,5,6,
	4,6,7,

	8,9,10,
	8,10,11,

	12,13,14,
	12,14,15,

	16,17,18,
	16,18,19,

	20,21,22,
	20,22,23
};