#include "House_Left.hpp"

#include <glm-0.9.5.2\glm\glm\gtc\matrix_transform.hpp>

using namespace cgue;
using namespace cgue::scene;

// Konstruktor
House_Left::House_Left(glm::mat4& matrix, Shader* _shader) : SceneObject(matrix), shader(_shader)
{
	// Load Data to buffer
	// pro attribut einen buffer

	// POSITION BUFFER
	glGenBuffers(1,&positionBuffer3);
	glBindBuffer(GL_ARRAY_BUFFER, positionBuffer3);
	glBufferData(GL_ARRAY_BUFFER, HOUSE_LEFT_VERTEX_COUNT * 3 * sizeof(float), positions, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	// NORMAL BUFFER
	glGenBuffers(1,&normalBuffer3);
	glBindBuffer(GL_ARRAY_BUFFER, normalBuffer3);
	glBufferData(GL_ARRAY_BUFFER, HOUSE_LEFT_VERTEX_COUNT * 3 * sizeof(float), normals, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	// UV BUFFER
	glGenBuffers(1,&uvBuffer3);
	glBindBuffer(GL_ARRAY_BUFFER, uvBuffer3);
	glBufferData(GL_ARRAY_BUFFER, HOUSE_LEFT_VERTEX_COUNT * 2 * sizeof(float), uvs, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	// INDEX BUFFER
	glGenBuffers(1,&indexBuffer3);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexBuffer3);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, HOUSE_LEFT_INDEX_COUNT * sizeof(unsigned int), indices, GL_STATIC_DRAW);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

	glGenVertexArrays(1, &vao);
	glBindVertexArray(vao);



	// Bind Buffers - Buffers auf eine Variable schreiben

	// POSITION BUFFER
	glBindBuffer(GL_ARRAY_BUFFER, positionBuffer3);
	GLint positionIndex = glGetAttribLocation(shader->programHandle, "position");
	glEnableVertexAttribArray(positionIndex);
	glVertexAttribPointer(positionIndex, 3, GL_FLOAT, GL_FALSE, 0, 0);

	// NORMAL BUFFER
	glBindBuffer(GL_ARRAY_BUFFER, normalBuffer3);
	GLint normalIndex = glGetAttribLocation(shader->programHandle, "normal");
	glEnableVertexAttribArray(normalIndex);
	glVertexAttribPointer(normalIndex, 3, GL_FLOAT, GL_FALSE, 0, 0);

	// UV BUFFER
	glBindBuffer(GL_ARRAY_BUFFER, uvBuffer3);
	GLint uvIndex = glGetAttribLocation(shader->programHandle, "uv");
	glEnableVertexAttribArray(uvIndex);
	glVertexAttribPointer(uvIndex, 2, GL_FLOAT, GL_FALSE, 0, 0);

	// INDEXBUFFER
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexBuffer3);

	glBindVertexArray(0);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	
}

House_Left::~House_Left() {
	
	glDeleteBuffers(1, &positionBuffer3);
	glDeleteBuffers(1, &indexBuffer3);
	glDeleteBuffers(1, &normalBuffer3);
	glDeleteBuffers(1, &uvBuffer3);
	glDeleteVertexArrays(1, &vao);
}

void House_Left::update(double deltaT, std::string tastatur) {
}

void House_Left::draw() {

	glBindVertexArray(vao);

	glDrawElements(GL_TRIANGLES, HOUSE_LEFT_INDEX_COUNT, GL_UNSIGNED_INT, 0);

	glBindVertexArray(0);
}

const float House_Left::positions[HOUSE_LEFT_VERTEX_COUNT * 3] = {


	// Back
	-16.0f, -4.5f, -10.0f,
	-16.0f, 10.5f, -10.0f,
	-15.0f, 10.5f, -10.0f,
	-15.0f, -4.5f, -10.0f,

	// Front
	-16.0f, -4.5f, 2.0f,
	-15.0f, -4.5f, 2.0f,
	-15.0f, 10.5f, -10.0f,
	-16.0f, 10.5f, 2.0f,

	//Top
	-15.0f, 10.5f, 2.0f,
	-15.0f, 10.5f, -10.0f,
	-16.0f, 10.5f, -10.0f,
	-16.0f, 10.5f, 2.0f,

	//Bottom
	-15.0f, -4.5f, 2.0f,
	-16.0f, -4.5f, 2.0f,
	-16.0f, -4.5f, -10.0f,
	-15.0f, -4.5f, -10.0f,

	//Right
	-15.0f, 10.5f, 2.0f,
	-15.0f, -4.5f, 2.0f,
	-15.0f, -4.5f, -10.0f,
	-15.0f, 10.5f, -10.0f,

	//Left
	-16.0f, 10.5f, 2.0f,
	-16.0f, 10.5f, -10.0f,
	-16.0f, -4.5f, -10.0f,
	-16.0f, -4.5f, 2.0f,
	
};


const float House_Left::normals[HOUSE_LEFT_VERTEX_COUNT * 3] = {
	
	0.0f, 1.0f, 0.0f,
	0.0f, 1.0f, 0.0f,
	0.0f, 1.0f, 0.0f,
	0.0f, 1.0f, 0.0f,

	0.0f, -1.0f, 0.0f,
	0.0f, -1.0f, 0.0f,
	0.0f, -1.0f, 0.0f,
	0.0f, -1.0f, 0.0f,

	1.0f, 0.0f, 0.0f,
	1.0f, 0.0f, 0.0f,
	1.0f, 0.0f, 0.0f,
	1.0f, 0.0f, 0.0f,

	-1.0f, 0.0f, 0.0f,
	-1.0f, 0.0f, 0.0f,
	-1.0f, 0.0f, 0.0f,
	-1.0f, 0.0f, 0.0f,

	0.0f, 0.0f, -1.0f,
	0.0f, 0.0f, -1.0f,
	0.0f, 0.0f, -1.0f,
	0.0f, 0.0f, -1.0f,

	0.0f, 0.0f, 1.0f,
	0.0f, 0.0f, 1.0f,
	0.0f, 0.0f, 1.0f,
	0.0f, 0.0f, 1.0f,
};

const float House_Left::uvs[HOUSE_LEFT_VERTEX_COUNT * 2] = {

	1.0f, 0.0f,
	1.0f, 1.0f,
	0.0f, 1.0f,
	0.0f, 0.0f,

	0.0f, 0.0f,
	1.0f, 0.0f,
	1.0f, 1.0f,
	0.0f, 1.0f,

	0.0f, 0.0f,
	1.0f, 0.0f,
	1.0f, 1.0f,
	0.0f, 1.0f,

	0.0f, 0.0f,
	1.0f, 0.0f,
	1.0f, 1.0f,
	0.0f, 1.0f,

	1.0f, 1.0f,
	1.0f, 0.0f,
	0.0f, 0.0f,
	0.0f, 1.0f,

	0.0f, 1.0f,
	1.0f, 1.0f,
	1.0f, 0.0f,
	0.0f, 0.0f

};

const unsigned int House_Left::indices[HOUSE_LEFT_INDEX_COUNT] = {

	0,1,2,
	0,2,3,

	4,5,6,
	4,6,7,

	8,9,10,
	8,10,11,

	12,13,14,
	12,14,15,

	16,17,18,
	16,18,19,

	20,21,22,
	20,22,23
};