#include "Street.hpp"

#include <glm-0.9.5.2\glm\glm\gtc\matrix_transform.hpp>

using namespace cgue;
using namespace cgue::scene;

// Konstruktor
Street::Street(glm::mat4& matrix, Shader* _shader) : SceneObject(matrix), shader(_shader)
{
	// Load Data to buffer
	// pro attribut einen buffer

	// POSITION BUFFER
	glGenBuffers(1,&positionBuffer2);
	glBindBuffer(GL_ARRAY_BUFFER, positionBuffer2);
	glBufferData(GL_ARRAY_BUFFER, STREET_VERTEX_COUNT * 3 * sizeof(float), positions, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	// NORMAL BUFFER
	glGenBuffers(1,&normalBuffer2);
	glBindBuffer(GL_ARRAY_BUFFER, normalBuffer2);
	glBufferData(GL_ARRAY_BUFFER, STREET_VERTEX_COUNT * 3 * sizeof(float), normals, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	// UV BUFFER
	glGenBuffers(1,&uvBuffer2);
	glBindBuffer(GL_ARRAY_BUFFER, uvBuffer2);
	glBufferData(GL_ARRAY_BUFFER, STREET_VERTEX_COUNT * 2 * sizeof(float), uvs, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	// INDEX BUFFER
	glGenBuffers(1,&indexBuffer2);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexBuffer2);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, STREET_INDEX_COUNT * sizeof(unsigned int), indices, GL_STATIC_DRAW);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);

	glGenVertexArrays(1, &vao);
	glBindVertexArray(vao);



	// Bind Buffers - Buffers auf eine Variable schreiben

	// POSITION BUFFER
	glBindBuffer(GL_ARRAY_BUFFER, positionBuffer2);
	GLint positionIndex = glGetAttribLocation(shader->programHandle, "position");
	glEnableVertexAttribArray(positionIndex);
	glVertexAttribPointer(positionIndex, 3, GL_FLOAT, GL_FALSE, 0, 0);

	// NORMAL BUFFER
	glBindBuffer(GL_ARRAY_BUFFER, normalBuffer2);
	GLint normalIndex = glGetAttribLocation(shader->programHandle, "normal");
	glEnableVertexAttribArray(normalIndex);
	glVertexAttribPointer(normalIndex, 3, GL_FLOAT, GL_FALSE, 0, 0);

	// UV BUFFER
	glBindBuffer(GL_ARRAY_BUFFER, uvBuffer2);
	GLint uvIndex = glGetAttribLocation(shader->programHandle, "uv");
	glEnableVertexAttribArray(uvIndex);
	glVertexAttribPointer(uvIndex, 2, GL_FLOAT, GL_FALSE, 0, 0);

	// INDEXBUFFER
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexBuffer2);

	glBindVertexArray(0);

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
}

Street::~Street() {
	
	glDeleteBuffers(1, &positionBuffer2);
	glDeleteBuffers(1, &indexBuffer2);
	glDeleteBuffers(1, &normalBuffer2);
	glDeleteBuffers(1, &uvBuffer2);
	glDeleteVertexArrays(1, &vao);
}

void Street::update(double deltaT, std::string tastatur) {
}

void Street::draw() {

	glBindVertexArray(vao);

	glDrawElements(GL_TRIANGLES, STREET_INDEX_COUNT, GL_UNSIGNED_INT, 0);

	glBindVertexArray(0);
}

const float Street::positions[STREET_VERTEX_COUNT * 3] = {


	// Back
	-500.0f, -5.5f, -600.0f,
	-500.0f, -4.5f, -600.0f,
	500.0f, -4.5f, -600.0f,
	500.0f, -5.5f, -600.0f,

	// Front
	-500.0f, -5.5f, 50.0f,
	500.0f, -5.5f, 50.0f,
	500.0f, -4.5f, 50.0f,
	-500.0f, -4.5f, 50.0f,

	//Top
	500.0f, -4.5f, 50.0f,
	500.0f, -4.5f, -600.0f,
	-500.0f, -4.5f, -600.0f,
	-500.0f, -4.5f, 50.0f,

	//Bottom
	500.0f, -5.5f, 50.0f,
	-500.0f, -5.5f, 50.0f,
	-500.0f, -5.5f, -600.0f,
	500.0f, -5.5f, -600.0f,

	//Right
	500.0f, -4.5f, 50.0f,
	500.0f, -5.5f, 50.0f,
	500.0f, -5.5f, -600.0f,
	500.0f, -4.5f, -600.0f,

	//Left
	-80.0f, -4.5f, 50.0f,
	-80.0f, -4.5f, -600.0f,
	-80.0f, -5.5f, -600.0f,
	-80.0f, -5.5f, 50.0f,
	
};


const float Street::normals[STREET_VERTEX_COUNT * 3] = {
	
	0.0f, 1.0f, 0.0f,
	0.0f, 1.0f, 0.0f,
	0.0f, 1.0f, 0.0f,
	0.0f, 1.0f, 0.0f,

	0.0f, -1.0f, 0.0f,
	0.0f, -1.0f, 0.0f,
	0.0f, -1.0f, 0.0f,
	0.0f, -1.0f, 0.0f,

	1.0f, 0.0f, 0.0f,
	1.0f, 0.0f, 0.0f,
	1.0f, 0.0f, 0.0f,
	1.0f, 0.0f, 0.0f,

	-1.0f, 0.0f, 0.0f,
	-1.0f, 0.0f, 0.0f,
	-1.0f, 0.0f, 0.0f,
	-1.0f, 0.0f, 0.0f,

	0.0f, 0.0f, -1.0f,
	0.0f, 0.0f, -1.0f,
	0.0f, 0.0f, -1.0f,
	0.0f, 0.0f, -1.0f,

	0.0f, 0.0f, 1.0f,
	0.0f, 0.0f, 1.0f,
	0.0f, 0.0f, 1.0f,
	0.0f, 0.0f, 1.0f,
};

const float Street::uvs[STREET_VERTEX_COUNT * 2] = {

	50.0f, 0.0f,
	50.0f, 50.0f,
	0.0f, 50.0f,
	0.0f, 0.0f,

	0.0f, 0.0f,
	50.0f, 0.0f,
	50.0f, 50.0f,
	0.0f, 50.0f,

	0.0f, 0.0f,
	50.0f, 0.0f,
	50.0f, 50.0f,
	0.0f, 50.0f,

	0.0f, 0.0f,
	50.0f, 0.0f,
	50.0f, 50.0f,
	0.0f, 50.0f,

	50.0f, 50.0f,
	50.0f, 0.0f,
	0.0f, 0.0f,
	0.0f, 50.0f,

	0.0f, 50.0f,
	50.0f, 50.0f,
	50.0f, 0.0f,
	0.0f, 0.0f

};

const unsigned int Street::indices[STREET_INDEX_COUNT] = {

	0,1,2,
	0,2,3,

	4,5,6,
	4,6,7,

	8,9,10,
	8,10,11,

	12,13,14,
	12,14,15,

	16,17,18,
	16,18,19,

	20,21,22,
	20,22,23
};