#version 330 core

in vec2 fUVs;

out vec3 color;

uniform sampler2D blurTexture;
uniform sampler2D depthBlurTexture;
uniform float deltaT;
uniform mat4 vpPrevious;
uniform mat4 vpInverse;

void main(){
	//color = texture( blurTexture, fUVs + 0.005*vec2( sin(time+1024.0*fUVs.x),cos(time+768.0*fUVs.y)) ).xyz ;
	//color = texture( blurTexture, fUVs).xyz ;

		int samples = 100;

		//Depth buffer value
		float zOverW = texture2D(depthBlurTexture, fUVs).r;
		//H = viewport position from -1 to 1
		vec4 H = vec4(fUVs.x * 2 - 1, fUVs.y * 2 - 1, zOverW, 1);
		//Transform by view-proj inverse
		vec4 D = H * vpInverse;
		//Divide by w to get world position
		vec4 worldPos = D/D.w;
		//Current viewport position
		vec4 currentPos = H;
		//Use the world position and transform by previous view-proj matrix
		//To see where this fragment was (on screen) last frame
		vec4 previousPos = worldPos * vpPrevious;
		//Convert to [-1,1] by dividing by w
		previousPos/=previousPos.w;
		//Use the difference in position to calculate pixel velocity
		vec3 velocity = ((currentPos - previousPos)/2.0).xyz;
		float mblurScale = (0.3/deltaT) / 60;
		velocity *= mblurScale;
		vec2 pixelVelocity = velocity.xy*2;
		pixelVelocity/=samples;
		//Get initial colour
		vec4 colour = texture(blurTexture, fUVs);

		vec2 currentTexCoord = fUVs;
		//Sample in direction of velocity
		for (int i=1; i<samples; ++i){
			currentTexCoord-=pixelVelocity;
			vec4 sampleColour = texture(blurTexture, currentTexCoord);
			colour += sampleColour;
		}

		//Average color
		vec4 finalColour = colour/samples;
		color = vec3(finalColour.rgb);





		/*int uBlurSize = 4;// use size of noise texture

		vec2 texelSize = 1.0 / vec2(textureSize(blurTexture, 0));
		float result = 0.0;
		vec2 hlim = vec2(float(-uBlurSize) * 0.5 + 0.5);
		for (int i = 0; i < uBlurSize; ++i) {
			for (int j = 0; j < uBlurSize; ++j) {
				vec2 offset = (hlim + vec2(float(i), float(j))) * texelSize;
				result += texture(blurTexture, fUVs + offset).r;
			}
		}
 
		color = result / float(uBlurSize * uBlurSize);*/

}