#version 330

in vec3 Position_worldspace;
in vec3 Normal_cameraspace;
in vec3 EyeDirection_cameraspace;
in vec3 LightDirection_cameraspace;
in vec4 ShadowCoord;

uniform vec3 lightPosition_worldspace;
uniform sampler2D colorTexture;
uniform sampler2DShadow shadowMap;
uniform float lightPower;
uniform float transparency;

in vec3 worldNormal;
in vec2 fUVs;
in float object; 

out vec4 color;

vec2 poissonDisk[16] = vec2[]( 
   vec2( -0.94201624, -0.39906216 ), 
   vec2( 0.94558609, -0.76890725 ), 
   vec2( -0.094184101, -0.92938870 ), 
   vec2( 0.34495938, 0.29387760 ), 
   vec2( -0.91588581, 0.45771432 ), 
   vec2( -0.81544232, -0.87912464 ), 
   vec2( -0.38277543, 0.27676845 ), 
   vec2( 0.97484398, 0.75648379 ), 
   vec2( 0.44323325, -0.97511554 ), 
   vec2( 0.53742981, -0.47373420 ), 
   vec2( -0.26496911, -0.41893023 ), 
   vec2( 0.79197514, 0.19090188 ), 
   vec2( -0.24188840, 0.99706507 ), 
   vec2( -0.81409955, 0.91437590 ), 
   vec2( 0.19984126, 0.78641367 ), 
   vec2( 0.14383161, -0.14100790 ) 
); 

 float exposure = 0.25;
		 float decay = 0.97;
		 float density = 0.97;
		 float weight = 0.5;
		uniform vec2 lightPositionOnScreen;
		uniform sampler2D firstPass;
		const int NUM_SAMPLES = 100;

		// Returns a random number based on a vec3 and an int.
 
float random(vec3 seed, int i){
 
	vec4 seed4 = vec4(seed,i);
 
	float dot_product = dot(seed4, vec4(12.9898,78.233,45.164,94.673));
 
	return fract(sin(dot_product) * 43758.5453);
 
} 


void main() {
		
		float bias = 0.005; 

		float visibility = 1.0;
		// Sample the shadow map 4 times
 
		for (int i=0;i<4;i++){
 
			// use either :
			//  - Always the same samples.
			//    Gives a fixed pattern in the shadow, but no noise
			int index = i;
			//  - A random sample, based on the pixel's screen location. 
			//    No banding, but the shadow moves with the camera, which looks weird.
			// int index = int(16.0*random(gl_FragCoord.xyz, i))%16;
			//  - A random sample, based on the pixel's position in world space.
			//    The position is rounded to the millimeter to avoid too much aliasing
			// int index = int(16.0*random(floor(Position_worldspace.xyz*1000.0), i))%16;
 
			// being fully in the shadow will eat up 4*0.2 = 0.8
			// 0.2 potentially remain, which is quite dark.
			visibility -= 0.2*(1.0-texture( shadowMap, vec3(ShadowCoord.xy/ShadowCoord.w + poissonDisk[index]/700.0,  (ShadowCoord.z-bias)/ShadowCoord.w) ));
		}

		vec3 lightColor = vec3(1, 1, 1);

        vec3 materialDiffuseColor = texture2D(colorTexture, fUVs).rgb;
        vec3 materialAmbientColor = vec3(0.1, 0.1, 0.1) * materialDiffuseColor;
        vec3 materialSpecularColor = vec3(0.3, 0.3, 0.3);

        float distance = length(lightPosition_worldspace - Position_worldspace);

        vec3 n = normalize(Normal_cameraspace);
        vec3 l = normalize(LightDirection_cameraspace);

        float cosTheta = clamp(dot(n, l), 0, 1);

        vec3 eye = normalize(EyeDirection_cameraspace);
        vec3 reflection = reflect(-l, n);

        float cosAlpha = clamp(dot(eye, reflection), 0, 1);

        vec3 color_temp = materialAmbientColor +
                visibility * materialDiffuseColor * lightColor * lightPower * cosTheta / (distance * distance) +
                visibility * materialSpecularColor * lightColor * lightPower * pow(cosAlpha, 5) / (distance * distance);
				
		color = vec4(color_temp, 1);
		color.a = transparency;
}